<?php
/**
 * Copyright (c) 2020. Cart2Quote B.V. All rights reserved.
 * See COPYING.txt for license details.
 */

namespace Cart2Quote\MageWorxCompatibility\Plugin;

use Magento\Framework\App\Request\Http;
use MageWorx\OptionBase\Model\ResourceModel\CollectionUpdaterRegistry;
use Magento\Catalog\Model\ResourceModel\Product\Option\Value\CollectionFactory as OptionValueCollectionFactory;
use Magento\Catalog\Model\ResourceModel\Product\Option\CollectionFactory as OptionCollectionFactory;
use Magento\Store\Model\StoreManagerInterface as StoreManager;
use MageWorx\OptionBase\Helper\System as SystemHelper;

class CollectProductConditions
{
    /**
     * @var CollectionUpdaterRegistry
     */
    private $collectionUpdaterRegistry;

    /**
     * @var OptionValueCollectionFactory
     */
    protected $optionValueCollectionFactory;

    /**
     * @var StoreManager
     */
    protected $storeManager;

    /**
     * @var SystemHelper
     */
    protected $systemHelper;

    /**
     * @var OptionCollectionFactory
     */
    protected $optionCollectionFactory;

    /**
     * @var Http
     */
    private $request;

    /**
     * @param CollectionUpdaterRegistry $collectionUpdaterRegistry
     * @param OptionValueCollectionFactory $optionValueCollectionFactory
     * @param SystemHelper $systemHelper
     * @param OptionCollectionFactory $optionCollectionFactory
     * @param StoreManager $storeManager
     * @param Http $request
     */
    public function __construct(
        CollectionUpdaterRegistry $collectionUpdaterRegistry,
        OptionValueCollectionFactory $optionValueCollectionFactory,
        SystemHelper $systemHelper,
        OptionCollectionFactory $optionCollectionFactory,
        StoreManager $storeManager,
        Http $request
    ) {
        $this->collectionUpdaterRegistry    = $collectionUpdaterRegistry;
        $this->optionValueCollectionFactory = $optionValueCollectionFactory;
        $this->systemHelper                 = $systemHelper;
        $this->optionCollectionFactory      = $optionCollectionFactory;
        $this->storeManager                 = $storeManager;
        $this->request                      = $request;
    }

    /**
     * Adding product custom options to result collection
     *
     * @return $this
     */
    public function aroundAddOptionsToResult($subject, \Closure $proceed)
    {
        $productsByLinkId = [];

        if ($this->request->getControllerModule() == "Cart2Quote_Quotation") {
            // If request is made by Cart2Quote do nothing
        } else {
            foreach ($subject as $product) {
                $productId = $product->getData(
                    $product->getResource()->getLinkField()
                );

                $productsByLinkId[$productId] = $product;
            }

            if (!empty($productsByLinkId)) {
                $this->collectionUpdaterRegistry->setOptionIds([]);
                $this->collectionUpdaterRegistry->setOptionValueIds([]);
                $this->collectionUpdaterRegistry->setCurrentEntityIds(array_keys($productsByLinkId));
                $this->collectionUpdaterRegistry->setCurrentRowIds(array_keys($productsByLinkId));

                $options = $this->optionCollectionFactory->create()->addTitleToResult(
                    $this->storeManager->getStore()->getId()
                )->addPriceToResult(
                    $this->storeManager->getStore()->getId()
                )->addProductToFilter(
                    array_keys($productsByLinkId)
                )->addValuesToResult();

                foreach ($options as $option) {
                    if (isset($productsByLinkId[$option->getProductId()])) {
                        $productsByLinkId[$option->getProductId()]->addOption($option);
                    }
                }
            }
        }

        return $subject;
    }
}
